@@ -9,7 +9,7 @@ from random import randint
 from sys import modules
 from typing import Dict, Optional
 
-from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, Discover
+from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, Discover, __dev_mode__
 from .entities import Entity, discover_decorator
 from .protocol import Command, Priority  # TODO: constants to const.py
 from .protocol.address import NON_DEV_ADDR, id_to_address
@@ -31,7 +31,7 @@ from .protocol.opentherm import MSG_ID, MSG_NAME, MSG_TYPE, OPENTHERM_MESSAGES,
 from .protocol.ramses import CODE_ONLY_FROM_CTL, RAMSES_DEVICES
 from .protocol.transport import PacketProtocolPort
 
-from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
     _0001,
     _0002,
@@ -342,7 +342,12 @@ class Device(DeviceInfo, DeviceBase):
         if not msg._gwy.config.enable_eavesdrop:
             return
 
-        if self._ctl is not None and "zone_idx" in msg.payload:
+        if (
+            self._ctl is not None
+            and "zone_idx" in msg.payload
+            and msg.src.type != "01"  # TODO: DEX, should be: if controller
+            # and msg.dst.type != "18"
+        ):
             # TODO: is buggy - remove? how?
             self._set_parent(self._ctl._evo._get_zone(msg.payload["zone_idx"]))
 
@@ -1447,7 +1452,6 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
                 if v
             },
             "opentherm_schema": self.opentherm_schema,
-            # "supported_msg": self._supported_msg,
         }
 
     @property
@@ -1455,12 +1459,11 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         return {
             **super().params,
             "opentherm_params": self.opentherm_params,
+            "supported_msgs": dict(sorted(self._supported_msg.items())),
         }
 
     @property
     def status(self) -> dict:
-        # llevering: [0, 3, 5, 6, 12, 13, 17, 18, 25, 26, 28, 48, 49, 56, 125]
-        # bruce:     [0, 3, 5,    12, 13, 17, 18, 25, 27, 28, 48, 49, 56, 125]
         return {
             **super().status,
             "opentherm_status": self.opentherm_status,
