@@ -43,9 +43,11 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _0B04,
     _1030,
     _1060,
+    _1081,
     _1090,
     _10A0,
     _10E0,
+    _10E1,
     _1100,
     _1260,
     _1280,
@@ -55,6 +57,8 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _12B0,
     _12C0,
     _12C8,
+    _12F0,
+    _1300,
     _1F09,
     _1F41,
     _1FC9,
@@ -76,6 +80,8 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     _31D9,
     _31DA,
     _31E0,
+    _3200,
+    _3210,
     _3220,
     _3B00,
     _3EF0,
