@@ -135,7 +135,7 @@ if DEV_MODE:
 def validate_command(has_zone=None):
     """Decorator to protect the engine from any invalid command constructors.
 
-    Additionally, validate/normalise some command arguments.
+    Additionally, validate/normalise some command arguments (e.g. 'HW' becomes 'FA').
     """
 
     def _wrapper(fcn, *args, **kwargs) -> Any:
@@ -281,6 +281,8 @@ class Command(PacketBase):
         self._rx_header = None
         self._tx_header = None
 
+        self._source_entity = None
+
     def __repr__(self) -> str:
         """Return an unambiguous string representation of this object."""
         hdr = f' # {self._hdr}{f" ({self._ctx})" if self._ctx else ""}'
