@@ -10,9 +10,9 @@ import logging
 import re
 from datetime import timedelta as td
 from functools import lru_cache
-from typing import Any, Optional, Tuple, Union
+from typing import Any, Optional, Tuple
 
-from .address import Address, dev_id_to_str
+from .address import Address
 from .exceptions import InvalidPacketError, InvalidPayloadError
 from .parsers import PAYLOAD_PARSERS, parser_unknown
 from .ramses import CODE_IDX_COMPLEX, CODE_RQ_COMPLEX, RAMSES_CODES, RAMSES_DEVICES
@@ -36,6 +36,7 @@ from .const import (  # noqa: F401, isort: skip
     _0404,
     _0418,
     _042F,
+    _0B04,
     _1030,
     _1060,
     _1090,
@@ -109,9 +110,8 @@ class Message:
         self._gwy = gwy
         self._pkt = pkt
 
-        # prefer Devices but can use Addresses...
-        self.src = gwy.device_by_id.get(pkt.src.id, pkt.src)
-        self.dst = gwy.device_by_id.get(pkt.dst.id, pkt.dst)
+        self.src = pkt.src
+        self.dst = pkt.dst
         self._addrs = pkt.addrs
 
         self.dtm = pkt.dtm
@@ -120,11 +120,10 @@ class Message:
         self.seqn = pkt.seqn
         self.code = pkt.code
         self.len = pkt.len
-        self.raw_payload = pkt.payload
 
         self.code_name = CODE_NAMES.get(self.code, f"unknown_{self.code}")
 
-        self._payload = self._validate(self.raw_payload)  # ? raise InvalidPacketError
+        self._payload = self._validate(self._pkt.payload)  # ? raise InvalidPacketError
 
         self._str = None
         self._expired_ = None
@@ -143,9 +142,10 @@ class Message:
                 return f"({pkt.payload[:2]})"
             return ctx
 
-        def display_name(dev: Union[Address, Any]) -> str:
-            name = dev.schema.get(ATTR_ALIAS) if self._gwy.config.use_aliases else None
-            return name[:20] if name else dev_id_to_str(dev.id)
+        def display_name(addr: Address) -> str:  # TODO: is broken
+            # name = d.schema.get(ATTR_ALIAS) if self._gwy.config.use_aliases else None
+            name = None
+            return name[:20] if name else Address._friendly(addr.id)
 
         if self._str is not None:
             return self._str
@@ -166,14 +166,12 @@ class Message:
     def __eq__(self, other) -> bool:
         if not isinstance(other, Message):
             return NotImplemented
-        return all(
-            (
-                self.verb == other.verb,
-                self.code == other.code,
-                self.src == other.src,
-                self.dst == other.dst,
-                self.raw_payload == other.raw_payload,
-            )
+        return (self.src, self.dst, self.verb, self.code, self._pkt.payload) == (
+            other.src,
+            other.dst,
+            other.verb,
+            other.code,
+            other._pkt.payload,
         )
 
     def __lt__(self, other) -> bool:
@@ -229,7 +227,14 @@ class Message:
             return {}
 
         #  I --- 00:034798 --:------ 12:126457 2309 003 0201F4
-        if not {self.src.type, self.dst.type} & {"01", "02", "12", "18", "22", "23"}:
+        if not {self.src.type, self.dst.type} & {
+            "01",
+            "02",
+            "12",
+            "18",
+            "22",
+            "23",
+        }:  # DEX
             assert self._pkt._idx == "00", "What!! (00)"
             return {}
 
@@ -239,7 +244,7 @@ class Message:
             "02",
             "18",
             "23",
-        ):
+        ):  # DEX
             assert self._pkt._idx == "00", "What!! (01)"
             return {}
 
@@ -260,7 +265,7 @@ class Message:
 
         if self.src.type == self.dst.type and not getattr(
             self.src, "_is_controller", True
-        ):
+        ):  # DEX
             assert self._pkt._idx == "00", "What!! (12)"
             return {}
 
@@ -301,7 +306,7 @@ class Message:
             ):
                 _logger = _LOGGER.info
             else:
-                _logger = _LOGGER.warning
+                _logger = _LOGGER.warning if DEV_MODE else _LOGGER.info
             _logger("%s # has expired %s", self._pkt, f"({self._expired_ * 100:1.0f}%)")
 
         # elif self._expired_ >= self.IS_EXPIRING:  # this could log multiple times
@@ -330,57 +335,56 @@ class Message:
 
         return self._is_fragment
 
-    def _validate(self, raw_payload) -> dict:
+    def _validate(self, raw_payload) -> Optional[dict]:  # TODO: needs work
         """Validate the message, and parse the payload if so.
 
         Raise an exception if it is not valid.
         """
 
-        # # STEP 0: Check verb/code pair against src/dst device type & payload
-        # if self.code not in RAMSES_CODES:
-        #     raise InvalidPacketError(f"Unknown code: {self.code}")
-
-        _check_verb_code_src(self)  # ? raise InvalidPacketError
-        _check_verb_code_dst(self)  # ? raise InvalidPacketError
-        _check_verb_code_payload(self, self.raw_payload)  # ? raise InvalidPayloadError
-
-        if not self._has_payload or (
-            self.verb == RQ and self.code not in CODE_RQ_COMPLEX
-        ):
-            # _LOGGER.error("%s", msg)
-            return {}
-
         try:  # parse the payload
+            _check_verb_code_payload(self, self._pkt.payload)  # ? InvalidPayloadError
+            _check_verb_code_src(self)  # ? InvalidPacketError
+            _check_verb_code_dst(self)  # ? InvalidPacketError
+
+            if not self._has_payload or (
+                self.verb == RQ and self.code not in CODE_RQ_COMPLEX
+            ):
+                # _LOGGER.error("%s", msg)
+                return {}
+
             result = PAYLOAD_PARSERS.get(self.code, parser_unknown)(
-                self.raw_payload, self
+                self._pkt.payload, self
             )
             assert isinstance(
                 result, (dict, list)
             ), f"invalid payload type: {type(result)}"
 
-        except AssertionError as err:
+        except AssertionError as exc:
             # beware: HGI80 can send parseable but 'odd' packets +/- get invalid reply
             (
                 _LOGGER.exception
-                if DEV_MODE and self.src.type != "18"
+                if DEV_MODE and self.src.type != "18"  # DEX
                 else _LOGGER.exception
-            )("%s << %s", self._pkt, f"{err.__class__.__name__}({err})")
+            )("%s << %s", self._pkt, f"{exc.__class__.__name__}({exc})")
 
-        except (InvalidPacketError, InvalidPayloadError) as err:  # CorruptEvohomeError
+        except InvalidPacketError as exc:
             (_LOGGER.exception if DEV_MODE else _LOGGER.warning)(
-                "%s << %s", self._pkt, err
+                "%s << %s", self._pkt, exc
             )
 
-        except (AttributeError, LookupError, TypeError, ValueError) as err:  # TODO: dev
+        except (AttributeError, LookupError, TypeError, ValueError) as exc:  # TODO: dev
             _LOGGER.exception(
-                "%s << Coding error: %s", self._pkt, f"{err.__class__.__name__}({err})"
+                "%s << Coding error: %s", self._pkt, f"{exc.__class__.__name__}({exc})"
             )
 
         except NotImplementedError:  # parser_unknown (unknown packet code)
             _LOGGER.warning("%s << Unknown packet code (cannot parse)", self._pkt)
 
-        # _LOGGER.error("%s", msg)
-        return result if isinstance(result, list) else {**self._idx, **result}
+        else:
+            # _LOGGER.error("%s", msg)
+            return result if isinstance(result, list) else {**self._idx, **result}
+
+        raise InvalidPacketError  # HACK: should be as PKT
 
 
 @lru_cache(maxsize=256)
@@ -393,18 +397,21 @@ def _check_verb_code_src(msg) -> None:
 
     Raise InvalidPacketError if the meta data is invalid, otherwise simply return.
     """
-    if msg.src.type not in RAMSES_DEVICES:
+    if msg.src.type not in RAMSES_DEVICES:  # DEX  # TODO: fingerprint dev class
         raise InvalidPacketError(f"Unknown src device type: {msg.src.id}")
 
-    if msg.src.type == "18":  # TODO: make a dynamic list if sensor/relay faking
+    if msg.src.type == "18":  # TODO: make a dynamic list if sensor/relay faking, DEX
         if msg.code not in RAMSES_CODES:  # NOTE: HGI can send whatever it likes
             raise InvalidPacketError(f"Unknown code for {msg.src.id} to Tx: {msg.code}")
         return
 
-    if msg.code not in RAMSES_DEVICES[msg.src.type]:
+    # if msg.dst.type == "18":  # TODO: make a dynamic list if sensor/relay faking, DEX
+    #     return
+
+    if msg.code not in RAMSES_DEVICES[msg.src.type]:  # DEX
         raise InvalidPacketError(f"Invalid code for {msg.src.id} to Tx: {msg.code}")
 
-    if msg.verb not in RAMSES_DEVICES[msg.src.type][msg.code]:
+    if msg.verb not in RAMSES_DEVICES[msg.src.type][msg.code]:  # DEX
         raise InvalidPacketError(
             f"Invalid verb/code for {msg.src.id} to Tx: {msg.verb}/{msg.code}"
         )
@@ -417,36 +424,36 @@ def _check_verb_code_dst(msg) -> None:
     """
 
     # check that the destination would normally respond...
-    if "18" in (msg.src.type, msg.dst.type) or msg.dst.type in ("--", "63"):
+    if "18" in (msg.src.type, msg.dst.type) or msg.dst.type in ("--", "63"):  # DEX
         return  # could omit this check to enforce strict checking
 
-    if msg.dst.type not in RAMSES_DEVICES:
+    if msg.dst.type not in RAMSES_DEVICES:  # DEX
         raise InvalidPacketError(f"Unknown dst device type: {msg.dst.id}")
 
-    if msg.verb == I_:  # receiving an I is not in the schema, so cant be tested
+    if msg.verb == I_:  # receiving an I isnt currently in the schema, so cant be tested
         return
 
-    # HACK: these exceptions need sorting
-    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"01/{RQ}/{_3EF1}",):
+    # HACK: these exceptions-to-the-rule need sorting
+    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"01/{RQ}/{_3EF1}",):  # DEX
         return
 
-    if msg.code not in RAMSES_DEVICES[msg.dst.type]:  # NOTE: is not OK for Rx
+    if msg.code not in RAMSES_DEVICES[msg.dst.type]:  # NOTE: is not OK for Rx, DEX
         #  I --- 04:253797 --:------ 01:063844 1060 003 056401
-        # HACK: these exceptions need sorting
+        # HACK: these exceptions-to-the-rule need sorting
         # if msg.code in (_1060, ):
         #     return
         raise InvalidPacketError(f"Invalid code for {msg.dst.id} to Rx: {msg.code}")
 
-    # HACK: these exceptions need sorting
+    # HACK: these exceptions-to-the-rule need sorting
     if f"{msg.verb}/{msg.code}" in (f"{W_}/{_0001}",):
         return
 
-    # HACK: these exceptions need sorting
-    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"13/{RQ}/{_3EF0}",):
+    # HACK: these exceptions-to-the-rule need sorting
+    if f"{msg.dst.type}/{msg.verb}/{msg.code}" in (f"13/{RQ}/{_3EF0}",):  # DEX
         return
 
     verb = {RQ: RP, RP: RQ, W_: I_}[msg.verb]
-    if verb not in RAMSES_DEVICES[msg.dst.type][msg.code]:
+    if verb not in RAMSES_DEVICES[msg.dst.type][msg.code]:  # DEX
         raise InvalidPacketError(
             f"Invalid verb/code for {msg.dst.id} to Rx: {msg.verb}/{msg.code}"
         )
@@ -459,16 +466,20 @@ def _check_verb_code_payload(msg, payload) -> None:
     Some parsers may also raise InvalidPayloadError (e.g. 3220).
     """
 
+    # if self.code not in RAMSES_CODES:
+    #     raise InvalidPacketError(f"Unknown code: {msg.code}")
+
     try:
         regex = RAMSES_CODES[msg.code][msg.verb]
-        if not re_compile_re_match(regex, payload):
-            raise InvalidPayloadError(f"{msg._pkt} < Payload doesn't match '{regex}'")
+        if not re_compile_re_match(regex, payload) and msg.src.type != "18":  # DEX
+            raise InvalidPayloadError(f"Payload doesn't match '{regex}': {payload}")
     except KeyError:
-        pass  # TODO: raise - missing entry in RAMSES schema
+        if "18" not in (msg.src.type, msg.dst.type):  # DEX
+            raise InvalidPacketError(f"Unknown verb/code pair: {msg.verb}/{msg.code}")
 
     # TODO: put this back, or leave it to the parser?
     # if msg.code == _3220:
-    #     msg_id = int(msg.raw_payload[4:6], 16)
+    #     msg_id = int(payload[4:6], 16)
     #     if msg_id not in OPENTHERM_MESSAGES:  # parser uses OTB_MSG_IDS
     #         raise InvalidPayloadError(
     #             f"OpenTherm: Unsupported data-id: 0x{msg_id:02X} ({msg_id})"
