@@ -12,9 +12,17 @@ from sys import modules
 from types import SimpleNamespace
 from typing import Optional
 
-from .const import DISCOVER_ALL, DISCOVER_PARAMS, DISCOVER_SCHEMA, DISCOVER_STATUS
-from .devices import BdrSwitch, Device, DhwSensor
-from .entities import Entity, discovery_filter
+from .const import Discover, __dev_mode__
+from .devices import (
+    BdrSwitch,
+    Controller,
+    Device,
+    DhwSensor,
+    Temperature,
+    TrvActuator,
+    UfhController,
+)
+from .entities import Entity, discover_decorator
 from .protocol import Command, Schedule
 from .protocol.const import (
     _000C_DEVICE,
@@ -31,16 +39,15 @@ from .protocol.const import (
     ATTR_WINDOW_OPEN,
     ATTR_ZONE_SENSOR,
     ATTR_ZONE_TYPE,
-    DEVICE_HAS_ZONE_SENSOR,
-    ZONE_CLASS_MAP,
     ZONE_MODE,
     ZONE_TYPE_MAP,
     ZONE_TYPE_SLUGS,
 )
 from .protocol.exceptions import CorruptStateError
+from .protocol.transport import PacketProtocolPort
 
 # from .ramses import RAMSES_ZONES, RAMSES_ZONES_ALL
-from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .protocol import I_, RP, RQ, W_  # noqa: F401, isort: skip
 from .protocol import (  # noqa: F401, isort: skip
     _0001,
     _0002,
@@ -142,6 +149,18 @@ class ZoneBase(Entity):
             return NotImplemented
         return self.idx < other.idx
 
+    def _start_discovery(self) -> None:
+
+        self._gwy._add_task(  # 0005/000C pkts
+            self._discover, discover_flag=Discover.SCHEMA, delay=1, period=3600 * 24
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.PARAMS, delay=3, period=3600 * 6
+        )
+        self._gwy._add_task(
+            self._discover, discover_flag=Discover.STATUS, delay=5, period=60 * 15
+        )
+
     def _set_system(self, parent, zone_idx):
         """Set the zone's parent system, after validating it."""
 
@@ -166,10 +185,9 @@ class ZoneBase(Entity):
 
         return parent, parent._ctl
 
-    def _send_cmd(self, code, **kwargs) -> None:
-        dest = kwargs.pop("dest_addr", self._ctl.id)
+    def _make_cmd(self, code, **kwargs) -> None:
         payload = kwargs.pop("payload", f"{self.idx}00")
-        super()._send_cmd(code, dest, payload, **kwargs)
+        super()._make_cmd(code, self._ctl.id, payload, **kwargs)
 
     @property
     def heating_type(self) -> Optional[str]:
@@ -178,29 +196,23 @@ class ZoneBase(Entity):
 
 
 class ZoneSchedule:  # 0404  # TODO: add for DHW
-    """Evohome zones have a schedule."""
-
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self._schedule = Schedule(self)
+        self._schedule = None
 
-    # @discovery_filter
-    # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-    #     if discover_flag & DISCOVER_STATUS:  # TODO: add back in
+    # def _discover(self, discover_flag=Discover.ALL) -> None:
+    #    super()._discover(discover_flag=discover_flag)
+
+    #     if discover_flag & Discover.STATUS:  # TODO: add back in
     #         self._loop.create_task(self.get_schedule())  # 0404
 
-    #     def _handle_msg(self, msg) -> bool:
+    #     def _handle_msg(self, msg) -> None:
     #     super()._handle_msg(msg)
 
     #     if msg.code == _0404 and msg.verb != RQ:
     #         _LOGGER.debug("Zone(%s): Received RP/0404 (schedule) pkt", self)
 
-    @property
-    def schedule(self) -> dict:
-        if self._schedule:
-            return self._schedule.schedule.get("schedule")
-
     async def get_schedule(self, force_refresh=None) -> Optional[dict]:
         await self._schedule.get_schedule(force_refresh=force_refresh)
         return self.schedule
@@ -209,6 +221,18 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
         schedule = {"zone_idx": self.idx, "schedule": schedule}
         await self._schedule.set_schedule(schedule)
 
+    @property
+    def schedule(self) -> dict:
+        if self._schedule is None:
+            try:
+                if packets := self._msgz[_0404][RP]:
+                    self._schedule = Schedule.create_from_pkts(self, packets.values())
+            except KeyError:
+                self._schedule = Schedule(self)
+
+        if self._schedule.schedule:
+            return self._schedule.schedule.get("schedule")
+
     @property
     def status(self) -> dict:
         return {
@@ -220,12 +244,11 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 class RelayDemand:  # 0008
     """Not all zones call for heat."""
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_STATUS:
-            self._send_cmd(_0008)  # , payload=self.idx
+        if discover_flag & Discover.STATUS:
+            self._send_cmd(Command.get_relay_demand(self._ctl.id, zone_idx=self.idx))
 
     @property
     def relay_demand(self) -> Optional[float]:  # 0008
@@ -268,34 +291,46 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         if htg_valve:
             self._set_htg_valve(htg_valve)
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         # super()._discover(discover_flag=discover_flag)
 
-        # if False and __dev_mode__ and self.idx == "FA":  # dev/test code
-        #     self.async_set_override(state="On")
+        if discover_flag & Discover.SCHEMA:
+            for dev_type in (
+                f"00{_000C_DEVICE.DHW_SENSOR}",
+                f"00{_000C_DEVICE.DHW}",
+                f"01{_000C_DEVICE.DHW}",
+            ):
+                try:
+                    _ = self._msgz[_000C][RP][dev_type]
+                except KeyError:
+                    self._make_cmd(_000C, payload=dev_type)
 
-        if discover_flag & DISCOVER_SCHEMA:
-            [  # 000C: find the DHW relay(s), if any, see: _000C_DEVICE_TYPE
-                self._send_cmd(_000C, payload=dev_type)
-                for dev_type in (
-                    f"00{_000C_DEVICE.DHW_SENSOR}",
-                    f"00{_000C_DEVICE.DHW}",
-                    f"01{_000C_DEVICE.DHW}",
-                )
-            ]
+        if discover_flag & Discover.PARAMS:
+            self._send_cmd(Command.get_dhw_params(self._ctl.id))
 
-        if discover_flag & DISCOVER_PARAMS:
-            self._gwy.send_cmd(Command.get_dhw_params(self._ctl.id))
+        if discover_flag & Discover.STATUS:
+            self._send_cmd(Command.get_dhw_mode(self._ctl.id))
+            self._send_cmd(Command.get_dhw_temp(self._ctl.id))
 
-        if discover_flag & DISCOVER_STATUS:
-            self._gwy.send_cmd(Command.get_dhw_mode(self._ctl.id))
-            self._gwy.send_cmd(Command.get_dhw_temp(self._ctl.id))
-
-    def _handle_msg(self, msg) -> bool:
+    def _handle_msg(self, msg) -> None:
         assert msg.src is self._ctl, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
+        if msg.code == _000C and msg.payload["devices"]:
+            LOOKUP = {
+                ATTR_DHW_SENSOR: self._set_sensor,
+                ATTR_DHW_VALVE: self._set_dhw_valve,
+                ATTR_DHW_VALVE_HTG: self._set_htg_valve,
+            }
+            devices = [
+                # self._gwy._get_device(d, ctl_id=msg.src.id, domain_id=...)
+                self._gwy._get_device(d)  # ctl_id=self._ctl.id, domain_id=self.idx)
+                for d in msg.payload["devices"]
+            ]
+
+            LOOKUP[msg.payload["device_class"]](devices[0])
+
     def _set_dhw_device(self, new_dev, old_dev, attr_name, dev_class, domain_id):
         if old_dev is new_dev:
             return old_dev
@@ -393,7 +428,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
 
     def set_mode(self, mode=None, active=None, until=None) -> Task:
         """Set the DHW mode (mode, active, until)."""
-        return self._gwy.send_cmd(
+        return self._send_cmd(
             Command.set_dhw_mode(self._ctl.id, mode=mode, active=active, until=until)
         )
 
@@ -417,7 +452,7 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         # if differential is None:
         #     setpoint = dhw_params["differential"]
 
-        return self._gwy.send_cmd(
+        return self._send_cmd(
             Command.set_dhw_params(self._ctl.id, setpoint, overrun, differential)
         )
 
@@ -473,64 +508,113 @@ class Zone(ZoneSchedule, ZoneBase):
         if sensor:
             self._set_sensor(sensor)
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    @discover_decorator  # NOTE: can mean is double-decorated
+    def _discover(self, discover_flag=Discover.ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
         # TODO: add code to determine zone type if it doesn't have one, using 0005s
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ALL}")
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ALL_SENSOR}")
+        if discover_flag & Discover.SCHEMA:
+            for dev_type in (_000C_DEVICE.ALL, _000C_DEVICE.ALL_SENSOR):
+                try:
+                    _ = self._msgz[_000C][RP][f"{self.idx}{dev_type}"]
+                except KeyError:
+                    self._make_cmd(_000C, payload=f"{self.idx}{dev_type}")
 
-        if discover_flag & DISCOVER_PARAMS:
-            self._gwy.send_cmd(Command.get_zone_config(self._ctl.id, self.idx))
-            self._gwy.send_cmd(Command.get_zone_name(self._ctl.id, self.idx))
+        if discover_flag & Discover.PARAMS:
+            self._send_cmd(Command.get_zone_config(self._ctl.id, self.idx))
+            self._send_cmd(Command.get_zone_name(self._ctl.id, self.idx))
 
-        if discover_flag & DISCOVER_STATUS:  # every 1h, CTL will not respond to a 3150
-            self._gwy.send_cmd(Command.get_zone_mode(self._ctl.id, self.idx))
-            self._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
-            self._gwy.send_cmd(Command.get_zone_window_state(self._ctl.id, self.idx))
+        if discover_flag & Discover.STATUS:  # every 1h, CTL will not respond to a 3150
+            self._send_cmd(Command.get_zone_mode(self._ctl.id, self.idx))
+            self._send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
+            self._send_cmd(Command.get_zone_window_state(self._ctl.id, self.idx))
 
         # start collecting the schedule
         # self._schedule.req_schedule()  # , restart=True) start collecting schedule
 
-    def _handle_msg(self, msg) -> bool:
-        assert msg.src is self._ctl and (
+    def _handle_msg(self, msg) -> None:
+        # if msg.code in _000C and msg.src.type == "02":  # DEX
+        #     if self._ufc is None
+        #         self._ufc = msg.src
+        #     else:
+        #         raise()
+
+        assert (msg.src is self._ctl or msg.src.type == "02") and (  # DEX
             isinstance(msg.payload, dict)
             or [d for d in msg.payload if d["zone_idx"] == self.idx]
         ), f"msg inappropriately routed to {self}"
 
-        assert msg.src is self._ctl and (
+        assert (msg.src is self._ctl or msg.src.type == "02") and (  # DEX
             isinstance(msg.payload, list) or msg.payload["zone_idx"] == self.idx
         ), f"msg inappropriately routed to {self}"
 
         super()._handle_msg(msg)
 
-        # not UFH (it seems), but ELE or VAL; and possibly a MIX support 0008 too
-        if msg.code in (_0008, _0009):  # TODO: how to determine is/isn't MIX?
-            assert msg.src.type in ("01", "13"), msg.src.type  # 01 as a stat
+        if msg.code == _000C and msg.payload["devices"]:
+
+            # TODO: testing this concept, hoping to learn device_id of UFC
+            if msg.payload["_device_class"] == _000C_DEVICE.UFH:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.UFH}")
+
+            devices = [
+                # self._gwy._get_device(d, ctl_id=msg.src.id, domain_id=...)
+                self._gwy._get_device(d)  # ctl_id=self._ctl.id, domain_id=self.idx)
+                for d in msg.payload["devices"]
+            ]
+
+            if msg.payload["_device_class"] == _000C_DEVICE.ALL_SENSOR:
+                self._set_sensor(devices[0])  # incl. devices[0]._set_parent(self)
+
+            elif msg.payload["_device_class"] == _000C_DEVICE.ALL:
+                [d._set_parent(self) for d in devices]  # if d is not None]
+
+        if msg._gwy.config.enable_eavesdrop and self._zone_type in (None, "ELE"):
+            self._eavesdrop_zone_type(msg)
+
+    def _eavesdrop_zone_type(self, msg, prev=None) -> None:
+        """TODO.
+
+        There are three ways to determine the type of a zone:
+        1. Use a 0005 packet (deterministic)
+        2. Eavesdrop (non-deterministic, slow to converge)
+        3. via a config file (a schema)
+        """
+        # ELE/VAL, but not UFH (it seems)
+        if msg.code in (_0008, _0009):
             assert self._zone_type in (None, "ELE", "VAL", "MIX"), self._zone_type
 
             if self._zone_type is None:
                 self._set_zone_type("ELE")  # might eventually be: "VAL"
 
-        elif msg.code == _30C9 and msg.verb in (I_, RP):  # used by sensor matching
-            assert msg.src.type in DEVICE_HAS_ZONE_SENSOR + ("01",), "coding error"
-
         elif msg.code == _3150:  # TODO: and msg.verb in (I_, RP)?
-            assert msg.src.type in ("00", "02", "04", "13")
-            assert self._zone_type in (None, "RAD", "UFH", "VAL")  # MIX/ELE don't 3150
+            # MIX/ELE don't 3150
+            assert self._zone_type in (None, "RAD", "UFH", "VAL"), self._zone_type
 
-            if msg.src.type in ("00", "02", "04", "13"):
-                zone_type = ZONE_CLASS_MAP[msg.src.type]
-                self._set_zone_type("VAL" if zone_type == "ELE" else zone_type)
+            if isinstance(msg.src, TrvActuator):
+                self._set_zone_type("RAD")
+            elif isinstance(msg.src, BdrSwitch):
+                self._set_zone_type("VAL")
+            elif isinstance(msg.src, UfhController):
+                self._set_zone_type("UFH")
 
     def _msg_value(self, *args, **kwargs):
         return super()._msg_value(*args, **kwargs, zone_idx=self.idx)
 
+    @property  # TODO:
+    def actuators(self) -> Device:
+        try:
+            return self._msgz["000C"]["RP"][f"{self.idx}00"].payload["devices"]
+        except LookupError:
+            pass
+
     @property
     def sensor(self) -> Device:
-        return self._sensor
+        try:
+            return self._gwy.device_by_id[
+                self._msgz["000C"]["RP"][f"{self.idx}04"].payload["devices"][0]
+            ]
+        except LookupError:
+            return self._sensor
 
     def _set_sensor(self, device: Device) -> None:  # self._sensor
         """Set the temp sensor for this zone (one of: 01:, 03:, 04:, 12:, 22:, 34:)."""
@@ -542,8 +626,7 @@ class Zone(ZoneSchedule, ZoneBase):
                 f"{self} changed {ATTR_ZONE_SENSOR}: {self._sensor} to {device}"
             )
 
-        sensor_types = ("00", "01", "03", "04", "12", "22", "34")
-        if not isinstance(device, Device) or device.type not in sensor_types:
+        if not isinstance(device, (Controller, Temperature)):
             # TODO: or not hasattr(device, "temperature")
             raise TypeError(f"{self}: {ATTR_ZONE_SENSOR} can't be: {device}")
 
@@ -552,37 +635,10 @@ class Zone(ZoneSchedule, ZoneBase):
 
     @property
     def heating_type(self) -> Optional[str]:
-        """TODO.
-
-        There are three ways to determine the type of a zone:
-        1. Use a 0005 packet (deterministic)
-        2. Eavesdrop (non-deterministic, slow to converge)
-        3. via a config file (a schema)
-        """
 
         if self._zone_type is not None:  # isinstance(self, ???)
             return ZONE_TYPE_MAP.get(self._zone_type)
 
-        # TODO: actuators
-        dev_types = [d.type for d in self.devices if d.type in ("00", "02", "04", "13")]
-
-        if "02" in dev_types:
-            zone_type = "UFH"
-        elif "13" in dev_types:
-            zone_type = "VAL" if _3150 in self._msgs else "ELE"
-        # elif "??" in dev_types:  # TODO:
-        #     zone_type = "MIX"
-        elif "04" in dev_types or "00" in dev_types:
-            # beware edge case: TRV as sensor for a non-RAD zone
-            zone_type = "RAD"
-        else:
-            zone_type = None
-
-        if zone_type is not None:
-            self._set_zone_type(zone_type)
-
-        return ZONE_TYPE_MAP.get(self._zone_type)
-
     def _set_zone_type(self, zone_type: str):  # self._zone_type
         """Set the zone's type, after validating it.
 
@@ -608,7 +664,7 @@ class Zone(ZoneSchedule, ZoneBase):
 
         self._zone_type = _type
         self.__class__ = ZONE_BY_TYPE[_type]
-        self._discover()  # TODO: needs tidyup (ref #67)
+        self._discover(discover_flag=Discover.ALL)  # TODO: needs tidyup (ref #67)
 
     @property
     def name(self) -> Optional[str]:  # 0004
@@ -618,19 +674,7 @@ class Zone(ZoneSchedule, ZoneBase):
     @name.setter
     def name(self, value) -> Optional[str]:
         """Set the name of the zone."""
-        self._gwy.send_cmd(Command.set_zone_name(self._ctl.id, self.idx, value))
-
-        # async def get_name(self, force_refresh=None) -> Optional[str]:
-        #     """Return the name of the zone."""
-        #     if not force_refresh:
-        #         return self.name
-
-        #     self._name = None
-        #     self._send_cmd(_0004, payload=f"{self.idx}00")
-        #     while self._name is None:
-        #         await asyncio.sleep(0.05)
-
-        #     return self.name
+        self._send_cmd(Command.set_zone_name(self._ctl.id, self.idx, value))
 
     @property
     def config(self) -> Optional[dict]:  # 000A
@@ -650,7 +694,7 @@ class Zone(ZoneSchedule, ZoneBase):
         if value is None:
             self.reset_mode()
         else:
-            self._gwy.send_cmd(Command.set_zone_setpoint(self._ctl.id, self.idx, value))
+            self._send_cmd(Command.set_zone_setpoint(self._ctl.id, self.idx, value))
 
     @property
     def temperature(self) -> Optional[float]:  # 30C9
@@ -679,7 +723,7 @@ class Zone(ZoneSchedule, ZoneBase):
 
     def _get_temp(self) -> Task:  # TODO: messy - needs tidy up
         """Get the zone's latest temp from the Controller."""
-        return self._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
+        return self._send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
 
     def reset_config(self) -> Task:  # 000A
         """Reset the zone's parameters to their default values."""
@@ -703,7 +747,7 @@ class Zone(ZoneSchedule, ZoneBase):
             openwindow_function=openwindow_function,
             multiroom_mode=multiroom_mode,
         )
-        return self._gwy.send_cmd(cmd)
+        return self._send_cmd(cmd)
 
     def reset_mode(self) -> Task:  # 2349
         """Revert the zone to following its schedule."""
@@ -719,11 +763,11 @@ class Zone(ZoneSchedule, ZoneBase):
             cmd = Command.set_zone_setpoint(self._ctl.id, self.idx, setpoint)
         else:
             cmd = Command.set_zone_mode(self._ctl.id, self.idx, mode, setpoint, until)
-        return self._gwy.send_cmd(cmd)
+        return self._send_cmd(cmd)
 
     def set_name(self, name) -> Task:
         """Set the zone's name."""
-        return self._gwy.send_cmd(Command.set_zone_name(self._ctl.id, self.idx, name))
+        return self._send_cmd(Command.set_zone_name(self._ctl.id, self.idx, name))
 
     @property
     def schema(self) -> dict:
@@ -744,6 +788,8 @@ class Zone(ZoneSchedule, ZoneBase):
             ATTR_ZONE_TYPE: self.heating_type,
             ATTR_ZONE_SENSOR: sensor_schema,
             ATTR_DEVICES: [d.id for d in self.devices],
+            "_sensor": self.sensor.id if self.sensor else None,
+            "_actuators": self.actuators,
         }
 
     @property  # TODO: setpoint
@@ -766,14 +812,18 @@ class EleZone(RelayDemand, Zone):  # BDR91A/T  # TODO: 0008/0009/3150
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't (can't) super()
-        # super()._discover(discover_flag=discover_flag)
-        if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ELE}")
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't (can't) super() initially
+        super()._discover(discover_flag=discover_flag)
 
-    def _handle_msg(self, msg) -> bool:
+        if discover_flag & Discover.SCHEMA:
+            try:
+                _ = self._msgz[_000C][RP][f"{self.idx}{_000C_DEVICE.ELE}"]
+            except KeyError:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ELE}")
+
+    def _handle_msg(self, msg) -> None:
         super()._handle_msg(msg)
 
         # if msg.code == _0008:  # ZON zones are ELE zones that also call for heat
@@ -799,16 +849,19 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't (can't) super()
-        # super()._discover(discover_flag=discover_flag)
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't (can't) super() initially
+        super()._discover(discover_flag=discover_flag)
 
-        if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.MIX}")
+        if discover_flag & Discover.SCHEMA:
+            try:
+                _ = self._msgz[_000C][RP][f"{self.idx}{_000C_DEVICE.MIX}"]
+            except KeyError:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.MIX}")
 
-        if discover_flag & DISCOVER_PARAMS:
-            self._gwy.send_cmd(Command.get_mix_valve_params(self._ctl.id, self.idx))
+        if discover_flag & Discover.PARAMS:
+            self._send_cmd(Command.get_mix_valve_params(self._ctl.id, self.idx))
 
     @property
     def mix_config(self) -> dict:  # 1030
@@ -829,13 +882,16 @@ class RadZone(Zone):  # HR92/HR80
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't (can't) super()
-        # super()._discover(discover_flag=discover_flag)
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't (can't) super() initially
+        super()._discover(discover_flag=discover_flag)
 
-        if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.RAD}")
+        if discover_flag & Discover.SCHEMA:
+            try:
+                _ = self._msgz[_000C][RP][f"{self.idx}{_000C_DEVICE.RAD}"]
+            except KeyError:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.RAD}")
 
 
 class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
@@ -845,13 +901,21 @@ class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't (can't) super()
-        # super()._discover(discover_flag=discover_flag)
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't (can't) super() initially
+        super()._discover(discover_flag=discover_flag)
 
-        if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.UFH}")
+        if discover_flag & Discover.SCHEMA:
+            try:
+                _ = self._msgz[_000C][RP][f"{self.idx}{_000C_DEVICE.UFH}"]
+            except KeyError:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.UFH}")
+
+    @property
+    def heat_demand(self) -> Optional[float]:  # 3150
+        """Return the zone's heat demand, estimated from its devices' heat demand."""
+        return self._msg_value(_3150, key=ATTR_HEAT_DEMAND)
 
     @property
     def ufh_setpoint(self) -> Optional[float]:  # 22C9
@@ -872,13 +936,16 @@ class ValZone(EleZone):  # BDR91A/T
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
-    @discovery_filter
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't (can't) super()
-        # super()._discover(discover_flag=discover_flag)
+    @discover_decorator
+    def _discover(self, discover_flag=Discover.ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't (can't) super() initially
+        super()._discover(discover_flag=discover_flag)
 
-        if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.VAL}")
+        if discover_flag & Discover.SCHEMA:
+            try:
+                _ = self._msgz[_000C][RP][f"{self.idx}{_000C_DEVICE.VAL}"]
+            except KeyError:
+                self._make_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.VAL}")
 
     @property
     def heat_demand(self) -> Optional[float]:  # 0008 (NOTE: not 3150)
@@ -904,17 +971,7 @@ def create_zone(evo, zone_idx, profile=None, **kwargs) -> Zone:
 
     zone = ZONE_BY_TYPE.get(profile, Zone)(evo, zone_idx, **kwargs)
 
-    if not evo._gwy.serial_port:
-        return zone
-
-    evo._gwy._add_task(
-        zone._discover, discover_flag=DISCOVER_SCHEMA, delay=2, period=86400
-    )
-    evo._gwy._add_task(
-        zone._discover, discover_flag=DISCOVER_PARAMS, delay=5, period=21600
-    )
-    evo._gwy._add_task(
-        zone._discover, discover_flag=DISCOVER_STATUS, delay=8, period=900
-    )
+    if isinstance(evo._gwy.pkt_protocol, PacketProtocolPort):
+        zone._start_discovery()
 
     return zone
